
/*
 *	iot.h
 *
 *	(C) Copyright 2003 by Robert Krten, all rights reserved.
 *	Please see the LICENSE file for more information.
 *
 *	This module contains the IOT decode table.
 *
 *	2003 12 16	R. Krten		created
*/

/*
 *	Conflicting decodes for IOTs
 *
 *	6050 - 6077		VC8I or KV8I; one only
 *	6440 - 6457		PT08 uses some of the data areas from the Data Communications System 680/I -- definition of PT08 is optional
 *	6530 - 6547		AF01A or AF04A; one only
 *	6551			used by AA01A, AA05, or not used.  Define zero or one only.
 *	6571			used by AF04A, AC01A, or not used.  Define zero or one only.
 *	6600 - 6667		used by DP01AA, overrides DF32/RF08 selection.  If defining DP01AA, don't bother with DF32/RF08
 *	6600 - 6627		used by DF32 and RF08, define one only.
 *	6640 - 6647		used by RF08 or not used, define RF08 or not.
 *	6700 - 6727		used by TC58 and TR02, define one only
*/

// Select one of the following:
#define	VC8I
//#define	KV8I

// Select PT08 if required; it will override some of the IOTs from the Data Communications System 680/I
//#define PT08

// Select one of the following:
#define	AF01A
//#define	AF04A

// Select one of the following:
#define	AA01A
//#define AA05

// If selecting DP01AA, don't bother with DF32/RF08
//#define	DP01AA
#ifndef	DP01AA
// Select one of the following:
#define	DF32
// #define RF08
#endif	// DP01AA

// Select one of the following:
#define	TC58
//#define TR02

static const char *iots [512] =
{
	// 6000			Interrupts
	"6000",
	"ION",
	"IOF",
	"6003",
	"6004",
	"6005",
	"6006",
	"6007",
	// 6010			High Speed Perforated Tape Reader and Control
	"6010",
	"RSF",			// 	Skip if reader flag is a 1.
	"RFB",			//	Read the content of the reader buffer and clear the reader flag. This instructions does not clear the AC.  RB v AC4-11 -> AC4-11
	"6013",
	"RFC",			//	Clear reader flag and reader buffer, fetch one character from tape and load it into the reader buffer, and set the reader flag when done.
	"6015",
	"6016",
	"6017",
	// 6020			High Speed Perforated Tape Punch and Control
	"6020",
	"PSF",			//	Skip if punch flag is a 1
	"PCF",			//	Clear punch flag and punch buffer.
	"6023",
	"PPC",			//	Load the punch buffer from bits 4 through 11 of the AC and punch the character.  This instructions does not clear the punch flag or punch buffer.  AC4-11 v PB -> PB
	"6025",
	"PLS",			//	Clear the punch flag, clear the bunch buffer, load the punch buffer from the content of bits 4 through 11 of the accumulator, punch the character, and set the punch flag to 1 when done.
	"6027",
	// 6030			Teletype Keyboard / Reader
	"6030",
	"KSF",			//	Skip if keyboard flag is a 1.
	"KCC",			//	Clear AC and clear keyboard flag.
	"6033",
	"KRS",			//	Read keyboard buffer static. This is a static command in that neither the AC nor the keyboard flag is cleared.  TTI v AC4-11 -> AC4-11
	"6035",
	"KRB",			//	Clear AC, clear keyboard flag, and read the content of the keyboard buffer into the content of AC4-11.
	"6037",
	// 6040			Teletype Teleprinter / Punch
	"6040",
	"TSF",			//	Skip if teleprinter flag is a 1.
	"TCF",			//	Clear teleprinter flag.
	"6043",
	"TPC",			//	Load the TTO from the content of AC4-11 and print and/or punch the character.
	"6045",
	"TLS",			//	Load the TTO from the content of AC4-11, clear the teleprinter flag, and print and/or punch the character.
	"6047",
#ifdef	VC8I
	// 6050			Oscilloscope Display Type VC8/I [VC8/L]
	"6050",
	"DCX",			//	Clear X coordinate buffer
	"6052",
	"DXL",			//	Clear and load X coordinate buffer.  AC2-11 -> YB
	"DIX",			//	Intensify the point defined by the content of the X and Y coordinate buffers.
	"6055",
	"6056",
	"DXS",			//	Executes the combined functions of DXL followed by DIX
	// 6060			(continued)
	"6060",
	"DCY",			//	Clear Y coordinate buffer
	"6062",
	"DYL",			//	Clear and load Y coordinate buffer.  AC2-11 -> YB
	"DIY",			//	Intensify the point defined by the content of the X and Y coordinate buffers.
	"6065",
	"6066",
	"DYS",			//	Executes the combined functions of DYL followed by DIY
	// 6070			(continued)
	"6070",
	"DSF",			//	(Light Pen Type 370) Skip if display flag is a 1.
	"DCF",			//	(Light Pen Type 370) Clear the display flag.
	"6073",
	"DSB",			//	Zero brightness
	"DSB",			//	Set minimum brightness
	"DSB",			//	Set medium brightness
	"DSB",			//	Set maximum brightness
#endif
#ifdef	KV8I
	// 6050			Storage Tube Display Control, Type KV8/I [KV8/L]
	"6050",
	"SNC",			//	Senses the condition of the cursor interrupt flag.  The flag produces an interrupt request when set by operation of the interrupt pushbutton on the joystick.  The flag is initially cleared when the computer is started.  As with all flag-sent instructions, SNC can be used under interrupt conditions to detect the source of the interrupt, or it can be used under interrupt on (ION) when the interrupt request has been caused by the operation of thecursor interrupt button.  In a program running with the interrupt off, SNC can be used to ignore the cursor successive approximation subroutine in the program if a request for service has not been made from the joystick controller.
	"CCF",			//	This instruction is used to clear the cursor flag after a request for service has been acknowledged by the program.
	"6053",
	"6054",
	"6055",
	"6056",
	"6057",
	// 6060			(continued)
	"6060",
	"6061",
	"SAC",			//	The analog comparator is set to compare the analog content of any one of six analog sources with the content of the digital-to-analog converter.  The analog sources are chosen according to a 3-bit binary code.  This code establishes the parameter for choosing the wanted register according to the content of AC2, AC3, and AC6.
	"LDF",			//	This instruction is used to establish the mode in which a wanted graphic is to be produced according to a 2-bit binary code.  This code determines whether the wanted vector will be linear absolute relative, whether the point plot mode will be used, or whether the cursor will be displayed.  This code establishes the paramteres for these formats according to the content of AC2 and AC3.  The LDF instruction must precede the LDX and LDY instructions.
	"LDX",			//	The X-axis sample and hold register is loaded with the binary equivalent of the X-axis coordinate according to the contents of AC2-11.  This data appears at the output of the digital-to-analog converter as the analog equivalent of the X-axis value of the binary word stored in the AC.  The LDX instruction clears an existing ready flag and sets the ready flag after 100 +/- 20 us.
	"LDY",			//	The Y-axis sample and hold register is loaded with the binary equivalent of the Y-axis coordinate according to the contents of AC2-11.  This data appears at the output of the digital-to-analog converter as the analog equivalent of the binary word in the AC.  The LDY instruction clears an existing ready flag and sets the ready flag after 100 +/- 20 us.
	"EXC",			//	Used to execute the wanted vector according to the contents of AC2-4 and AC6-11.  The parameter word establishes long or short formats, circular vectors, display erasure, reset of the integrators, and intensification of the vector.  The EXC instruction clears an existing ready flag and sets the ready flag as follows:  a) after 20 +/- 5 us for a point or vector continue; b) after 250 us for short vectors; c) after 4.05 ms for long vectors; d) after 500 ms for an erase.
	"6067",
	// 6070			(continued)
	"6070",
	"SRF",			//	Used to determine when the controller is ready to perform the next execute instruction.  The ready flag produces an interrupt condition when set.  The flag can be set by pressing the erase pushbutton on the VT01 unit.  Normally, however, the state of this flag is determined by the controller.  This flag is initially cleared when the computer is started and prior to an LDX, LDY, or EXC instruction.
	"CRF",			//	This instruction clears the ready flag after a skip instruction has been acknowledged.
	"SDA",			//	Used in the successive approximation subroutine to determine the digital equivalent of the selected analog holding register.  This instruction is used with the SAC (6062) instruction.
	"LDA",			//	This instruction is used to load the content of AC2-11.  This instruction is used with DSA (6073) in the successive approximation subroutine to determine the digital value of the content of the selected analog holding register.  Does not change flag states.
	"6075",
	"6076",
	"6077",
#endif
	// 6100			Memory Parity Type MP8/I [MP8/L]
	// 6100			Automatic Restart Type KP8/I [KP8/L]
	"6100",
	"SMP",			//	(MP8/I) Skip if memory parity error flag = 0.
	"SPL",			//	(KP8/I) Skip if power low
	"6103",
	"CMP",			//	(MP8/I) Clear memory parity error flag.
	"6105",
	"6106",
	"6107",
	// 6110			Multiple Asynchronous Serial Line Interface Unit, Type DC02D
	"6110",
	"MKSF",			//	Skip the next instruction if the keyboard flag is set.
	"MKCC",			//	Clear the keyboard and reader flags; clear the AC.
	"MTPF",			//	(DC02A) Transfer status of teleprinter flags to AC 0-3.
	"MKRS",			//	Transfer the shift register contents to AC 4-11.
	"MINT",			//	(DC02A) Interrupt on if AC 11 is set (interrupt request, if any flags).
	"MKRB",			//	Clear the keyboard and reader flags, clear the AC, trasnfer the shift register contents to AC 4-11 (MKCC and MKRS combined).
	"MTON",			//	Transfer AC0-3 to selection register (SELF) (select stations when bit is set).
	// 6120			Multiple Asynchronous Line Unit, Type DC02A
	"6120",
	"MTSF",			//	Skip the next instruction if the teleprinter flag is set.
	"MTCF",			//	Clear the teleprinter flag.
	"MTKF",			//	Transfer status of keyboard flags to AC 0-3.
	"MTPC",			//	Load AC4-11 into the shift register (begin print/punch).
	"MINS",			//	Skip if the interrupt request is active (if interrupt is on and any flag is raised).
	"MTLS",			//	Clear the teleprinter flag and load AC4-11 into the shift register (MTCF and MTPC combined)
	"MTRS",			//	Trasnfer the status of the selection register to AC 0-3.
	// 6130			Real Time Clock, Type KW8/I [KW8/L]
	"6130",
	"6131",
	"CCFF",			//	The flag, flag buffer, clock enable, and interrupt enable flip-flops are cleared.  This disables the real-time clock.
	"CSCF",			//	When the flag flip-flop has been set by a clock pulse, the flag buffer flip-flop is set to a 1.  Upon execution of this instruction, an IO BUS IN SKIP is generated if the flag is set.  The content of the PC is incremented by 1, so that the next sequential instruction is skipped.  The flag flip-flop is then cleared.  If the flag flip-flop has not been set, no skip is generated nor is the flag flip-flop cleared.
	"CRCA",			//	The output buffer is gated to the I/O BUS during IOP4, and a CLK AC CLR signal generated.  This register contains the last count in the count register.  The transfer from the count register is synchronized with this instruction so that a transfer that would occur during this instruction is not made.
	"6135",
	"CCEC",			//	All clock control flip-flops are first cleared, then the clock enable flip-flop is set.  For the variable frequency clock, the frequency source is enabled synchronously with program operation.  With all clocks, the data input to the flag is enabled after IOP2 time.  This represents an 800-ns mask, after the clock is enabled.
	"CECI",			//	All clock control flip-flops are cleared, then the clock enable, and interrupt enable flip-flops are set.  The clock enable flip-flop is described with the CCEC instruction.  The interrupt enable flip-flop allows an IO BUS IN INT signal when the flag is set.
	// 6140
	"6140",
	"6141",
	"6142",
	"6143",
	"6144",
	"6145",
	"6146",
	"6147",
	// 6150
	"6150",
	"6151",
	"6152",
	"6153",
	"6154",
	"6155",
	"6156",
	"6157",
	// 6160
	"6160",
	"6161",
	"6162",
	"6163",
	"6164",
	"6165",
	"6166",
	"6167",
	// 6170
	"6170",
	"6171",
	"6172",
	"6173",
	"6174",
	"6175",
	"6176",
	"6177",
	// 6200 through 6277	Memory Extension Control Type MC8/I [MC8/L]
	"6200",
	"CDF0",				//	Change to data field 0. The data field register is loaded with the selected field number (0).  All subsequent memory requests for operands are automatically switched to that data field until the data field number is changed by a new CDF command.
	"CIF0",				//	Prepare to change to instruction field N (0).  The instruction buffer register is loaded with the selected field number (0).  The next JMP or JMS instruction causes the new field to be entered.
	"6203",
	"CINT",				//	(KT8/I) Clear user interrupt.  Resets the user interrupt (UINT) flip-flop to the 0 state.
	"6205",
	"6206",
	"6207",
	// 6210
	"6210",
	"CDF1",				//	See CDF0 above
	"CIF1",				//	See CIF0 above
	"6213",
	"RDF",				//	Read data field into AC6-8.  Bit 0-5 and 9-11 of the AC are not affected.
	"6215",
	"6216",
	"6217",
	// 6220
	"6220",
	"CDF2",				//	See CDF0 above
	"CIF2",				//	See CIF0 above
	"6223",
	"RIF",				//	Same as RDF except reads the instruction field
	"6225",
	"6226",
	"6227",
	// 6230
	"6230",
	"CDF3",				//	See CDF0 above
	"CIF3",				//	See CIF0 above
	"6233",
	"RIB",				//	Read interrupt buffer.  The instruction field and data field stored during an interrupt are read into AC6-8 and AC9-11, respectively.
	"6235",
	"6236",
	"6237",
	// 6240
	"6240",
	"CDF4",				//	See CDF0 above
	"CIF4",				//	See CIF0 above
	"6243",
	"RMF",				//	Restore memory field.  Used to exit from a program interrupt.
	"6245",
	"6246",
	"6247",
	// 6250
	"6250",
	"CDF5",				//	See CDF0 above
	"CIF5",				//	See CIF0 above
	"6253",
	"SINT",				//	(KT8/I) Skip on user interrupt.  When the user interrupt (UINT) flip-flop is in the 1 state, sets the user skip flip-flop (USF) to the 1 state and causes the program to skip the next instruction.
	"6255",
	"6256",
	"6257",
	// 6260
	"6260",
	"CDF6",				//	See CDF0 above
	"CIF6",				//	See CIF0 above
	"6263",
	"CUF",				//	(KT8/I) Clears the user flag.  Clears the user buffer (UB) flip-flop.
	"6265",
	"6266",
	"6267",
	// 6270
	"6270",
	"CDF7",				//	See CDF0 above
	"CIF7",				//	See CIF0 above
	"6273",
	"SUF",				//	(KT8/I) Sets the user flag.  Sets user buffer (UB) and inhibits processor interrupts until the next JMP or JMS instruction.  Generation of IB -> IF during the next JMP or JMS instruction transfers the state of UB to the user field (UF) flip-flop.
	"6275",
	"6276",
	"6277",
	// 6300
	"6300",
	"6301",
	"6302",
	"6303",
	"6304",
	"6305",
	"6306",
	"6307",
	// 6310
	"6310",
	"6311",
	"6312",
	"6313",
	"6314",
	"6315",
	"6316",
	"6317",
	// 6320
	"6320",
	"6321",
	"6322",
	"6323",
	"6324",
	"6325",
	"6326",
	"6327",
	// 6330
	"6330",
	"6331",
	"6332",
	"6333",
	"6334",
	"6335",
	"6336",
	"6337",
	// 6340
	"6340",
	"6341",
	"6342",
	"6343",
	"6344",
	"6345",
	"6346",
	"6347",
	// 6350
	"6350",
	"6351",
	"6352",
	"6353",
	"6354",
	"6355",
	"6356",
	"6357",
	// 6360
	"6360",
	"6361",
	"6362",
	"6363",
	"6364",
	"6365",
	"6366",
	"6367",
	// 6370
	"6370",
	"6371",
	"6372",
	"6373",
	"6374",
	"6375",
	"6376",
	"6377",
	// 6400			Data Communications System 680/I
	"6400",
	"TTINCR",		//	This instruction causes the contents of the line register to be incremented by 1.  This command, when microprogrammed with a TTO command is executed.
	"TTI",			//	Causes a JMS to be executed (N+3) if the R register does not equal 0 and either the line hold bit of the selected line (specified by bits 2-8 of the LSW) is in the 1 state, or as a result of jamming the line state into and shifting the CAW; bit 11 of the CAW is a 1.
	"6403",
	"TTO",			//	Clears the link and shifts the link and accumulator one bit position to the right.  Bit 11 of the accumulator is shifted into the line unit specified by the line register.  The previuos contents (1 bit) of the selected line unit is lost.
	"6405",
	"6406",
	"6407",
	// 6410
	"6410",
	"TTCL",			//	The command sets the contents of the line register to 0.
	"TTSL",			//	The contents of AC5-11 are ORed into the line register.
	"TTLL",			//	The contents of AC5-11 are trasnferred into the line register.  This is a microprogram of TTCL and TTSL.
	"TTRL",			//	The contents of the line register are ORed into AC5-11.  The AC must be 0 for a true transfer.
	"6415",
	"6416",
	"6417",
	// 6420
	"6420",
	"T1skip",		//	(Data Communications System 680/I) Clock Control Instruction:  Causes the program to skip the next instruction if clock flag 1 is in the 1 state.  To clear the flag, either T!on or T!off can be used.
	"T1off",		//	(Data Communications System 680/I) Clock Control Instruction:  Inhibits clock 1 from setting its flag.  This instruction also sets the flag to the 0 state.
	"6423",
	"T1on",			//	(Data Communications System 680/I) Clock Control Instruction:  Enables clock 1 to set its flag at the predetermined lcock rate.  The flag in the 1 state causes a program interrupt when the interrupt is enabled.  This instruction also sets the flag to the 0 state.
	"6425",
	"6426",
	"6427",
	// 6430
	"6430",
	"T2skip",		//	(Data Communications System 680/I) same as T1skip except for clock 2
	"T2off",		//	(Data Communications System 680/I) same as T1off except for clock 2
	"6433",
	"T2on",			//	(Data Communications System 680/I) same as T1on except for clock 2
	"6435",
	"6436",
	"6437",
#ifdef	PT08
	// 6440			Asynchronous Serial Line Interface, Type PT08
	"6440",
	"TSFXXX",		//	Skip if teleprinter/punch 3 flag is a 1.
	"TCFXXX",		//	Clear teleprinter/punch 3 flag.
	"6443",
	"TPCXXX",		//	Load teleprinter 3 buffer (TTOX) with AC4-11 and print/punch the character.
	"6445",
	"TLSXXX",		//	Load TTOX with AC4-11, 3 flag, print/punch the character and clear teleprinter/punch.
	"6447",
	// 6450
	"6450",
	"KSFXXX",		//	Skip if keyboard/reader 3 flag is a 1.
	"KCCXXX",		//	Clear AC and keyboard/reader 3 flag.
	"6453",
	"KRSXXX",		//	Read keyboard/reader 3 buffer (TTI3) static.  TTI3 is loaded into AC4-11 by an OR transfer.
	"6455",
	"KRBXXX",		//	Clear the AC, read TTI3 into AC4-11, and clear keyboard 3 flag.
	"6457",
#else	// PT08
	"6440",
	"T3skip",		//	(Data Communications System 680/I) same as T1skip except for clock 3
	"T3off",		//	(Data Communications System 680/I) same as T1off except for clock 3
	"6443",
	"T3on",			//	(Data Communications System 680/I) same as T1on except for clock 3
	"6445",
	"6446",
	"6447",
	"6450",
	"T4skip",		//	(Data Communications System 680/I) same as T1skip except for clock 4
	"T4off",		//	(Data Communications System 680/I) same as T1off except for clock 4
	"6453",
	"T4on",			//	(Data Communications System 680/I) same as T1on except for clock 4
	"6455",
	"6456",
	"6457",
#endif
	// 6460
	"6460",
	"TTRINC",		//	(Data Communications System 680/I) This command causes the contents of the R register to be incremented by 1.  Because it is loaded with a 2's complement number, the result is a subtract.  This instruction can be microprogrammed with TTRR.
	"6462",
	"6463",
	"TTRR",			//	(Data Communications System 680/I) This command reads the contents of the R register into AC7-11.  The contents of the AC must be 0s before issuing this instruction.  This instruction, when microprogrammed with TTINCR, causes the incremented results to be read into the AC.
	"6465",
	"6466",
	"6467",
	// 6470
	"6470",
	"TTCR",			//	(Data Communications System 680/I) This command causes the R register to be set to 0
	"TTLR",			//	(Data Communications System 680/I) This command causes the contents of AC7-11 to be trasnferred into the R register.
	"6473",
	"6474",
	"6475",
	"6476",
	"6477",
	// 6500			Incremental Plotter and Control Type VP8/I
	"6500",
	"PLSF",			//	Skip if plotter flag is a 1.
	"PLCF",			//	Clear plotter flag.
	"6503",
	"PLPU",			//	Plotter pen up.  Raise pen off of paper.
	"6505",
	"6506",
	"6507",
	// 6510
	"6510",
	"PLPR",			//	Plotter pen right.
	"PLDU",			//	Plotter drum (paper) upward
	"6513",
	"PLDD",			//	Plotter drum (paper) downward.
	"6515",
	"6516",
	"6517",
	// 6520
	"6520",
	"PLPL",			//	Plotter pen left.
	"PLUD",			//	Plotter drum (paper) upward. Same as 6512
	"6523",
	"PLPD",			//	Plotter pen down.  Lower pen on to paper.
	"6525",
	"6526",
	"6527",
#ifdef	AF01A
	// 6530			General Purpose Converter and Multiplexer Control Type AF01A (this option is mutually exclusive with AF04A)
	"6530",
	"ADSE",			//	Skip if A/D converter flag is a 1.
	"ADCV",			//	Clear A/D converter flag and convert input voltage to a digital number, flag will set to 1 at end of conversion.  Number of bits in converted number determined by switch setting, 11 bits maximum.
	"6533",
	"ADRB",			//	Read A/D converter buffer into AC, left justified, and clear flag.
	"6535",
	"6536",
	"6537",
	// 6540
	"6540",
	"ADCC",			//	Clear multiplexer channel address register.
	"ADSC",			//	Set up multiplexer channel as per AC6-11.  Maximum of 64 single ended or 32 differntial input channels.
	"6543",
	"ADIC",			//	Index multiplexer channel address (present address + 1).  Upon reaching address limit, increment will cause channel 00 to be selected.
	"6545",
	"6546",
	"6547",
#endif	// AF01A
#ifdef	AF04A
	// 6530			Guarded Scanning Digital Voltmeter Type AF04A (this option is mutually exclusive with AF01A)
	"6530",
	"VSDR",			//	Skip if data ready flag is a 1.
	"VRD",			//	Selected byte of voltmeter is transferred to the accumulator and the data ready flag is cleared.
	"6533",
	"VBA",			//	BYTE ADVANCE command requests next twelve bits, data ready flag is set.
	"6535",
	"6536",
	"6537",
	// 6540
	"6540",
	"VCNV",			//	The contents of the accumulator are transferred to the AF04A channel address register.  Analog signal on selected channel is automatically digitized.
	"VSEL",			//	The contents of the accumulator are transferred to the AF04A control register.
	"6543",
	"VINX",			//	The last channel address is incremented by one and the analog signal on the selected channel is automatically digitized.
	"6545",
	"6546",
	"6547",
#endif	// AF04A
	// 6550
	"6550",
#ifdef	AA01A
	"DAL1",			//	(AA01A) The character in the accumulator is loaded into the channel 1 buffer.  The DAC then converts the buffered value to the analog equivalent.  NOTE: Similar instructions for DAL2 and DAL3 load respective DACs.
#else	// AA01A
#ifdef	AA05
	"CLDA",			//	The address register in the AA05/AA07 is cleared.
#else	// AA05
	"6551",
#endif	// AA05
#endif	// AA01A
	"LDAD",			//	(AA05/AA07) The address register in the AA05/AA07 is loaded with the contents of AC0-5.
	"6553",
	"6554",
	"6555",
	"6556",
	"6557",
	// 6560
	"6560",
	"6561",
	"LDAR",			//	(AA05/AA07) The buffer (input buffer, if the channel is double-buffered) of the DAC is loaded from AC0-9.
	"6563",
	"UPDT",			//	(AA05/AA07) The contents of the input buffers of all double-buffered channels are trasnferred to their respective output buffers.  The input buffer is not affected by this instruction.
	"6565",
	"6566",
	"6567",
	// 6570
	"6570",
#ifdef	AF04A
	"VSCC",			//	SAMPLE CURRENT CHANNEL when required to digitize analog signal on current channel repeatedly (AF04A)
#else	// AF04A
#ifdef	AC01A
	"HRAN",			//	The contents of AC3-5 are trasnferred to the channel address register (CHAR).  The 3-bit code is decoded to address any of the 8 channels.
#else	// AC01A
	"6571",
#endif	// AC01A
#endif	// AF04A
#ifdef	AC01A
	"HSIM",			//	Simultaneously places all 8 channels into the hold mode.
#else	// AC01A
	"6572",
#endif	// AC01A
	"6573",
	"SAMP",			//	(AA01A) Places all 8 channels into the sample (or track) mode.
	"6575",
	"6576",
	"6577",
#ifdef	DP01AA
	// 6600			Synchronous Modem Interface, Type DP01AA
	"6600",
	"TAC",			//	Causes the contents of the AC (6, 7, 8, or 9 bits right-justified) to be transferred into the TB.
	"CTF",			//	Resets the trasnmit flag.  If trasnmit active flag is set, CTF also causes the program to skip the next instruction.
	"6603",
	"CIM",			//	Resets the transmit logic idle mode (IM) flip-flop.
	"6605",
	"6606",
	"6607",
	"6610",
	"STF",			//	Causes the program to skip the next instruction if the transmit flag is in the 0 state.  When the transmit flag is in the 1 state, the trasnmit buffer register (TB) is ready to accept another character.
	"RRB",			//	Transfers the contents of the receiver buffer (RB) (6, 7, 8, or 9 bits, right-justified) to the computer AC.  RRB also resets the receive flag.
	"6613",
	"SIM",			//	Sets the transmit idle mode (IM) flip-flop.
	"6615",
	"6616",
	"6617",
	"6620",
	"SEF",			//	Causes the program to skip the next instruction if the receive end flag is 0.  The receive end flag slip-flop is set when the receive logic has stopped receiving serial data from the communications equipment due to termination of th SERIAL CLOCK RECEIVE pulse train.
	"CEF",			//	Clears the receive end flag
	"6623",
	"SRE",			//	Sets the ring enable (RE) flip-flop to a 1, which permits the ring flag to request a program interrupt.
	"6625",
	"6626",
	"6627",
	"6630",
	"SRI",			//	Causes the program to skip the next instruction if the ring flag is 0.  The ring flag is set when a ring input is received.
	"CRF",			//	Clears the ring flag.
	"6633",
	"STR",			//	Sets the terminal read (TR) flip-flop to the 1 state.  This causes the terminal ready lead to the modem to be set on the ON state.  The state changes to OFF for CTR
	"6635",
	"6636",
	"6637",
	"6640",
	"SSR",			//	Causes the program to skip the next instruction if the data-set-ready lead from the modem is in the ON state.
	"CTR",			//	Clears the terminal ready (TR) flip-flop (see STR)
	"6643",
	"CRE",			//	Clears the ring enable (RE) flip-flop.
	"6645",
	"6646",
	"6647",
	"6650",
	"SRF",			//	Causes the program to skip the next instruction if the receive flag is 0.  The flag is set when a received character is ready for trasnfer to the AC and the flag is cleared when an RRB instruction is issued.
	"CRA",			//	Clears the receive active (RA) flip-flop, taking the receive logic out of the active state.  This inhibits any more receive flags until a new sync character is received.
	"6653",
	"XOB",			//	Causes an exclusive OR of the AC with the buffer register (BR).
	"6655",
	"6656",
	"6657",
	"6660",
	"COB",			//	Clears the XOR buffer.
	"ROB",			//	Transfers the buffer register (BR) content to the AC.
	"6663",
	"IOB",			//	Transfers 1s from the AC to the buffer register (BR).
	"6665",
	"6666",
	"6667",
#else	// DP01AA
#ifdef	DF32
	// 6600			Random Access Disc File (type DF32)
	"6600",
	"DCMA",			//	Clears memory address register, parity erorr and completion flags.  This instruction clears the disk memory request flag and interrupt flags.
	"6602",
	"DMAR",			//	The contents of the AC are loaded into the disk memory address register and the AC is cleared.  Begin to read information from the disk into the specified core location.  Clears parity error and completion flags.  Clears interrupt flags.
	"6604",
	"DMAW",			//	The contents of the AC are loaded into the disk memory address register and the AC is cleared.  Begin to write information into the disk from the specified core location.  Clears parity error and completion flags.
	"6606",
	"6607",
	// 6610
	"6610",
	"DCEA",			//	Clears the disk extended address and memory address extension register.
	"DSAC",			//	Skips next instruction if address confirmed flag is a 1.  AC is cleared.
	"6613",
	"6614",
	"DEAL",			//	The disk extended-address extension registers are cleared and loaded with the track data held in the AC.
	"DEAC",			//	Clear the AC then loads the contents of the disk extended-address register into the AC to allow program evaluation.  Skip next instruction if address confirmed flag is a 1.
	"6617",
	// 6620
	"6620",
	"DFSE",			//	Skip next instruction if the completion flag is a 1.  Indicates data transfer is complete.
	"6622",
	"6623",
	"6624",
	"6625",
	"DMAC",			//	Clear the AC then loads contents of disk memory address register into the AC to allow program evaluation.
	"6627",
#endif	// DF32
#ifdef	RF08
	// 6600			Disk File and Control, Type RF08/Expander Disk File, Type RS08
	"6600",
	"6601",
	"6602",
	"6603",
	"6604",
	"6605",
	"6606",
	"6607",
	// 6610
	"6610",
	"DCIM",			//	Clear the disk interrupt enable and core memory address extension register.
	"6612",
	"6613",
	"6614",
	"DIML",			//	Clear the interrupt enable and memory address extension register, then load the interrupt enable and memory address extension registers with data held in the AC.  Then clear the AC.  NOTE: Transfers cannot occur across memory fields.  Attempts to do so will cause the transfer to "wrap around" within the specified memory field.
	"DIMA",			//	Clear the AC.  Then load the contents of the status register (STR) into the AC to allow program evaluation.
	"6617",
	// 6620
	"6620",
	"DFSE",			//	Skip next instruction if there is a parity error, data request late, write lock status, or nonexistent disk flag set.
	"6622",
	"DISK",			//	If either the error or data completion flag (or both) is set, the next instruction is skipped.
	"6624",
	"6625",
	"6626",
	"6627",
#endif	// RF08
	// 6630			Card Reader and Control Type CR8/I [CR8/L] (see also 6670)
	"6630",
	"RCSF",			//	Generates an IOP pulse (IOP 1) to test the data-ready flag output.  If the data ready flag is 1, the next sequential program instruction is skipped.
	"RCRA",			//	Generates an IOP pulse (IOP 2) to read the alphanumeric data at the control-logic buffer register and clear the data ready flag.
	"6633",
	"RCRB",			//	Generates an IOP pulse (IOP 4) to read the BCD data at the control logic buffer register and clear the data ready flag.
	"6635",
	"6636",
	"6637",
#ifdef	RF08
	// 6640
	"6640",
	"DCXA",			//	Clear the high order 8-bits of the disk address register (DAR).
	"6642",
	"DXAL",			//	Clear the high order 8 bits of the DAR.  Then load the DAR from data stored in the AC.  Then clear the AC.
	"6644",
	"DXAC",			//	Clear the AC; then load the contents of the high order 8-bit DAR into the AC.
	"DMMT",			//	For maintenance purposes only with the appropriate maintenance cable connections and the disk disconnected from the RS08 logic, the (given) standard signals may be generated by IOT 6646 and associated AC bits.  The AC is cleared and the maintenance register is initiated by issuing an IOT 6601 command.
	"6647",
#else	// RF08
	// 6640
	"6640",
	"6641",
	"6642",
	"6643",
	"6644",
	"6645",
	"6646",
	"6647",
#endif	// RF08
	// 6650			Automatic Line Printer and Control Type 645
	"6650",
	"LSE",			//	Skip if line printer error flag is a 1.
	"LCB",			//	Clear both sections of the printing buffer.
	"6653",
	"LLB",			//	Load printing buffer from the content of AC6-11 and clear the AC
	"6655",
	"6656",
	"6657",
	// 6660			Automatic Line Printer and Control Type 645 (continued)
	"6660",
	"LSD",			//	Skip if the printer done flag is a 1.
	"LCF",			//	Clear line printer done and error flags.
	"6663",
	"LPR",			//	Clear the format register, load the format register from the content of AC9-11, print the line contained in the section of the printer buffer loaded last, clear the AC, and advance the paper in accordance with the selected channel of the format tape if the content of AC8=1.  If the content of AC8=0, the line is printed and paper advance is inhibited.
	"6665",
	"6666",
	"6667",
#endif	// DP01AA
	// 6670			Card Reader and Control Type CR8/I [CR8/L] (see also 6630)
	"6670",
	"RCSD",			//	Generates an IOP pulse (IOP 1) to test the card-done flag output.  If the card done flag is 1, the next sequential program instruction is skipped.
	"RCSE",			//	Generates an IOP pulse (IOP 2) to advance the card, clear the card done flag, and produce a skip flag is reader is ready.  If skip flag is generated, the next sequential program instruction is skipped.
	"6673",
	"RCRD",			//	Generates an IOP pulse (IOP 4) to clear the card done flag.
	"6675",
	"6676",
	"6677",
#ifdef	TC58
	// 6700			Automatic Magnetic Tape Control Type TC58
	"6700",
	"MTSF",			//	Skip on error flag or magnetic tape flag.  The status of the error flag (EF) and the magnetic tape flag (MTF) are sampled.  If either or both are set to 1, the content of the PC is incremented by one to skip the next sequential instruction.
	"6702",			//	(no mnemonic assigned) Clear the accumulator.
	"6703",
	"6704",			//	(no mnemonic assigned) Inclusively OR the contents of the status register into AC0-11
	"6705",
	"MTRS",			//	Read the contents of the status register into AC0-11.
	"6707",
	// 6710
	"6710",
	"MTCR",			//	Skip on tape control ready (TCR).  If the tape control is ready to receive a command, the PC is incremented by one to skip the next sequential instruction.
	"MTAF",			//	Clear the status and command registers, and the EF and MTF if tape control ready.  If tape control not ready, clears MTF and EF flags only.
	"6713",
	"MTCM",			//	Inclusively OR the contents of AC0-5, AC9-11 into the command register; JAM transfer bits 6, 7, 8 (command function)
	"6715",
	"MTLC",			//	Load the contents of AC0-1j1 into the command register.
	"6717",
	// 6720
	"6720",
	"MTTR",			//	Skip on tape transport ready (TTR).  The next sequential instruction is skipped if the tape transport is ready.
	"MTGO",			//	Set "go" bit to execute command in the command register if command is legal.
	"6723",
	"MTRC",			//	Inclusively OR the contents of the contents of the command register into AC0-11.
	"6725",
	"6726",
	"6727",
#endif	// TC58
#ifdef	TR02
	// 6700			Incremental Magnetic Tape Controller, Type TR02
	"6700",
	"IRS",			//	When data is ready to be strobed into the AC from the read buffer (RB), the PC is incremented by one to skip the next sequential instruction.  The read done flag is cleared only if the skip occurs.
	"ISR",			//	The content of the status register (STR) is read into AC0-8.  The AC should be cleared before it is read by this instruction.
	"IWS",			//	If the write done flag is set, the next instruction is skipped and the write done flag is cleared.
	"IMC",			//	The move command decoded from AC0-2 is generated.  This instruction also clears the read done, write done, and gap detect flags.  The indicated flag is set when the command has been executed.
	"IGS",			//	If the gap detect flag is set, the next instruction is skipped and the gap detect flag is cleared.
	"IWR",			//	The contents of the AC are loaded into the tape input data buffer (WB) and a write step command is generated.  The write done flag is set when writing is completed.
	"IRD",			//	The AC is cleared and the content of the read buffer (RB) is loaded into the AC.  Data bits are transferred into AC6-11 (7-track) or AC4-11 (9-track).  Parity error is transferred into AC0 which is 0 if there is no parity error.
	// 6710
	"6710",
	"IRSA",			//	When data is ready to be strobed into the AC from the read buffer (RB), the PC is incremented by one to skip the next sequential instruction.  The read done flag is cleared only if the skip occurs.
	"ISRA",			//	The content of the status register (STR) is read into AC0-8.  The AC should be cleared before it is read by this instruction.
	"IWSA",			//	If the write done flag is set, the next instruction is skipped and the write done flag is cleared.
	"IMCA",			//	The move command decoded from AC0-2 is generated.  This instruction also clears the read done, write done, and gap detect flags.  The indicated flag is set when the command has been executed.
	"IGSA",			//	If the gap detect flag is set, the next instruction is skipped and the gap detect flag is cleared.
	"IWRA",			//	The contents of the AC are loaded into the tape input data buffer (WB) and a write step command is generated.  The write done flag is set when writing is completed.
	"IRDA",			//	The AC is cleared and the content of the read buffer (RB) is loaded into the AC.  Data bits are transferred into AC6-11 (7-track) or AC4-11 (9-track).  Parity error is transferred into AC0 which is 0 if there is no parity error.
	// 6720
	"6720",
	"6721",
	"6722",
	"6723",
	"6724",
	"6725",
	"6726",
	"6727",
#endif	// TC58
	// 6730
	"6730",
	"6731",
	"6732",
	"6733",
	"6734",
	"6735",
	"6736",
	"6737",
	// 6740
	"6740",
	"6741",
	"6742",
	"6743",
	"6744",
	"6745",
	"6746",
	"6747",
	// 6750
	"6750",
	"6751",
	"6752",
	"6753",
	"6754",
	"6755",
	"6756",
	"6757",
	// 6760			DECtape Transport Type TU55 and DECtape Control Type TC01
	"6760",
	"DTRA",			//	The content of status register A is read into AC0-9 by an OR transfer.  The bit assignments are:  AC0-2 = Transport unit select numnber; AC3-4 = Motion; AC5 = Mode; AC6-8 = Function; AC9 = Enable/disable DECtape control flag.
	"DCTA",			//	Clear status register A.  All flags undisturbed.
	"6763",
	"DTXA",			//	Status register A is loaded by an exclusive OR transfer from the content of the AC, and AC10 and AC11 are sampled.  If AC10 = 0, the error flags are cleared.  If AC11 = 0, the DECtape control flag is cleared.
	"6765",
	"6766",
	"6767",
	// 6770
	"6770",
	"DTSF",			//	Skip if error flag is a 1 or if DECtape control flag is a 1.
	"DTRB",			//	The content of status register B is read into the AC by an OR transfer.  The bit assignments are:  AC0 = Error flag; AC1 = Mark track error; AC2 = End of tape ; AC3 = Select error ; AC4 = Parity error; AC5 = Timing error; AC6-8 = Memory field; AC9-10 = Unused; AC11 = DECtape flag.
	"6773",
	"DTLB",			//	The memory field portion of status register B is loaded from the content of AC6-8.
	"6775",
	"6776",
	"6777"
};

